%		function [R AllCors] = corr_relf(spiketimes, sigma,mode,ini,fin,allpairs)
%								spiketimes=N x 1 x P	(N=number of trials=number of cells, P=spike times in secs, zero padded)
%								sigma = gaussian width (secs). for single spike precision, sigma~=0.01 (10ms)
%               		ini, fin: window of analysis in secs
%               		allpairs=1 => all pairs
%								allpairs=0 => Consecutive pairs (good approximation of allpairs).
%								mode: Just for graphic purposes
% calculates the cosine-correlation type reliability measure - Analytical version (with help from Peter Thomas)
%			This is the analytical version of convolutions with gaussian kernels of width sigma.
% R=reliability, AllCors=matrix of correlations/similarities
%
%Jean-Marc Fellous (fellous@salk.edu)


function [R, AllCors] = corr_relf(stimes,sigma, mode,ini,fin,allpairs)

%global waves nwaves pulse npulses ncurrent ppulses pcurrent range base labels maxT waves2 nwaves2

ntrials=size(stimes,1);
srate=10000;				% sampling rate (10 kHz)
sig=sigma*srate;

R=0;
AllCors=eye(ntrials,ntrials);

if allpairs==1
	% ---all pairs
    for i=1:ntrials-1
    	    tmp=stimes(i,1,:);ts=tmp(:);idx=find(ts>ini & ts<fin);s1=floor(ts(idx)*srate);
	    	for j=i+1:ntrials
		    	tmp=stimes(j,1,:);ts=tmp(:);idx=find(ts>ini & ts<fin);s2=floor(ts(idx)*srate);
		    	if ~isempty(s1) & ~isempty(s2)
		    		AllCors(i,j) = prodct(s1,s2,sig)/sqrt(prodct(s1,s1,sig)*prodct(s2,s2,sig));
            	else
                    AllCors(i,j)=0;
            	end
		    	R=R+AllCors(i,j);
        	end
    end
	R=R*2/(ntrials*(ntrials-1));
	type='All pairs';
else
    %---consecutive pairs only
    for i=1:ntrials-1
	    j=i+1;
	    tmp=stimes(i,1,:);ts=tmp(:);idx=find(ts>ini & ts<fin);s1=floor(ts(idx)*srate);
	    tmp=stimes(j,1,:);ts=tmp(:);idx=find(ts>ini & ts<fin);s2=floor(ts(idx)*srate);
	    if ~isempty(s1) & ~isempty(s2)
	    	AllCors(i,j) = prodct(s1,s2,sig)/sqrt(prodct(s1,s1,sig)*prodct(s2,s2,sig));
	    else
	        AllCors(i,j)=0;
	    end
	    R=R+AllCors(i,j);	
    end
    R=R/(ntrials-1);
    type='Consecutive pairs';
end

if (mode >0)
	clf
	hold on
	greycols(255);
    imagesc(AllCors)	
    axis('image')
	title(sprintf('%s - Reliability:%.2f',type,R));
	drawnow
end


% Analytical version - sigma in sample points

function s=prodct(s1,s2, sigma)

sigma2 = sigma^2; % shorthand
sigmapi = 2*sigma*sqrt(pi); % shorthand
gg=0;
for a = 1:length(s1)
	gg = gg + sum(exp(-((s2 - s1(a)).^2)/(4*sigma2)));
end
s=gg/sigmapi;
			
			